from typing import Type
from arc import ExecutionError

from ..types import OpenContext
from .dmenu_like import Dmenu, Rofi, RofiQL, Wofi, WofiQL
from .ui_provider import UIProvider


ProviderType = dict[str, dict[OpenContext, Type[UIProvider]]]
providers: ProviderType = {
    "rofi": {
        OpenContext.BROWSE: Rofi,
        OpenContext.QUICK_LAUNCH: RofiQL,
    },
    "wofi": {
        OpenContext.BROWSE: Wofi,
        OpenContext.QUICK_LAUNCH: WofiQL,
    },
    "dmenu": {
        OpenContext.BROWSE: Dmenu,
        OpenContext.QUICK_LAUNCH: Dmenu,
    },
}


def get_ui_provider(provider_str: str, open_context: OpenContext) -> Type[UIProvider]:
    if provider_str not in providers:
        raise ExecutionError(
            f"{provider_str} is not a valid UI Provider. "
            f" Valid providers: {providers.keys()}"
        )
    if open_context not in providers[provider_str]:
        raise ExecutionError(f"{provider_str} does not support {open_context.value}")

    return providers[provider_str][open_context]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['venture', 'venture.ui']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'arc-cli>=3.0.0,<4.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'ujson>=4.0.2,<5.0.0']

entry_points = \
{'console_scripts': ['venture = venture.cli:cli']}

setup_kwargs = {
    'name': 'venture',
    'version': '2.1.0',
    'description': 'Rofi / Wofi based project selector',
    'long_description': '# Venture\n\nA Dmenu / Rofi / Wofi menu to open projects and files in your favorite editor!\n\n\n\n### Dependancies\n- Venture supports three UI providers: dmenu, rofi, and wofi. It is expected that you have the one you intend to use installed.\n\n- Venture uses [Nerdfonts](https://www.nerdfonts.com/) for icons\n\n## Installation\n\n```\n$ pip install venture\n```\n\n## Docs\n[Here](./docs)\n\nNote this may installing into `~/.local/bin` which is not part of the default path on some Linux systems',
    'author': 'Sean Collings',
    'author_email': 'seanrcollings@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/seanrcollings/venture',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations
from typing import Iterable
import subprocess
from .ui_provider import UIProvider
from .. import util
from ..icons import icon_map, iconize, Icon
from ..browse_list import BrowseItem


class DmenuLike(UIProvider[dict[str, BrowseItem], str]):
    command: str
    default_args: Iterable[str] = []
    argument_format: str
    seperator: str = "\n"

    def run(self) -> str | None:
        args = self.get_commandline_args()
        output = self.execute(args)
        parsed = self.parse_output(output)
        return self._display_items.get(parsed)

    def format_items(self, items: dict[str, BrowseItem]):
        return {
            f"{self.get_icon(item['icon'])} {name}": item["path"]
            for name, item in items.items()
        }

    def get_commandline_args(self) -> Iterable[str]:
        if not self.config.args:
            return []

        return [
            item
            for name, value in self.config.args.items()
            for item in self.format_arg(name, value)
        ]

    def format_arg(self, name, value):
        if value == False:
            return []
        if value == True:
            value = ""

        return (
            self.argument_format.format(name=name, value=value).rstrip(" ").split(" ")
        )

    def execute(self, args):
        """Execute the UI command.

        This implementation expects the UI to be an external process
        and for it to recieve it's input as a newline-seperated
        list of strings from stdin.
        """
        with subprocess.Popen(
            [self.command, *self.default_args, *args],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        ) as proc:
            return proc.communicate(
                bytes(self.seperator.join(self._display_items), "utf-8")
            )[0]

    def parse_output(self, output: bytes):
        choice = output.decode("utf-8").strip("\n")
        return choice

    def get_icon(self, icon: Icon | list[str]):
        if not self.config.browse.show_icons:
            return ""

        code, color = icon
        string = f"{code:<2}"

        if self.config.color_icons:
            string = util.pango_span(string, color=color)

        return string


class Dmenu(DmenuLike):
    command = "dmenu"
    argument_format = "--{name} {value}"


class Rofi(DmenuLike):
    command = "rofi"
    seperator = "|"
    default_args = ["-dmenu", "-markup-rows", "-sep", seperator]
    argument_format = "-{name} {value}"


class QL(DmenuLike):
    tag_sep: str = "\n"

    def format_items(self, items):
        return {
            f"{iconize(item.get('icon', ''), self.config.color_icons):<2} "
            f"{name}{self.tag_sep}{self.format_tags(item)}": item
            for name, item in items.items()
        }

    def format_tags(self, item):
        tags = [iconize(tag, self.config.color_icons) for tag in item.get("tags", [])]

        all_tags = (
            [f"{icon_map.get('directory').code}  {item['path']}"] + tags
            if self.config.quicklaunch.show_filepath
            else tags
        )
        return util.pango_span(
            ", ".join(all_tags),
            color="#9c9c9c",
            size="smaller",
            weight="light",
        )


class RofiQL(Rofi, QL):
    default_args = Rofi.default_args + ["-eh", "2"]


class Wofi(DmenuLike):
    command = "wofi"
    default_args = ["--dmenu", "--allow-markup"]
    argument_format = "--{name} {value}"


class WofiQL(Wofi, QL):
    tag_sep = " "
